* Functions to dump a database to Excel via DDE
*
*
* Usage:
*    set procedure to l:\tyler\scripts\exceldump additive
*    * Open up the database or make a cursor or something
*    DumpToExcel()
*
* DumpToExcel() takes many parameters -- see below

set procedure to l:\tyler\scripts\exceldde.prg additive


* DumpToExcel(dbName, Row, Mc, FormatString, InitFunc, IncreFunc, DiffFunc, SkipTitles)
* 
* Dumps a database to Excel via DDE.  Can dump tables, cursors, etc.
* May switch to the specified database, but will switch back to the one that you
* started in.
*
* dbName = If specified, dump that database instead.  If not specified or if .F.,
*          dump the current database, if one is open.
* Row    = Starting row for the headers.  If not specified, or if it is .F.,
*          it will start on row 1
* Mc     = DDE connection returned by StartExcelDDE().  If not specified, or
*          if it is .F., this will create a new connection
* FormatString = How you want the columns formatted.  See below.
* InitFunc = Name of a function to call to initialize counter variables
* IncreFunc = Name of a function to call to increment counter variables
* DiffFunc = Name of a function to call that takes appropriate action for differences
* SkipTitles = Skip writing titles to the Excel spreadsheet
*
* Returns the DDE connection to Excel, so you can continue to alter the spreadsheet.
*
* FormatString is a string that describes how you want the data formatted.
* The first letter corresponds to the first column.  The second letter is for
* the second column, and so forth.  So, if your columns are num_claims, venname,
* amount, percent, you would want the FormatString to be "0 $2" so that the
* first column has zero decimal places, the second one is auto-formatted,
* the third is currency ($), and the percent column shows two decimal places.
*
*    Character    Explanation
*    ---------    ------------------------------------------------------
*    space        Auto-formatted
*    0-9          Zero to 9 decimal places with commas
*    $            Currency with commas and two decimal places
*    [otherwise]  Auto-formatted (treats unknown characters like space)
*
* InitFunc, IncreFunc, and DiffFunc are provided to allow subtotals and other nifty
* things to be displayed.  For instance, if you want to sum the field named cbamnt
* whenever vennum changed, you could do something like this:
*
*   function my_init
*      old_vennum = vennum
*      cbsum = 0
*   endfunc
* 
*   function my_increment
*      cbsum = cbsum + cbamnt
*   endfunc
*
*   function my_difference
*   parameters mc, r
*      if eof() or old_vennum != vennum
*         if ! eof()
*            old_vennum = vennum
*         endif
*         m.r = m.r + 1  && Move to next line
*         DDEPoke(mc, MakeRC(r, 1), "Subtotal:")
*         DDEPoke(mc, MakeRC(r, 2), allt(str(cbsum))
*         cbsum = 0
*         return m.r + 1 && Move again to next line for start of next data chunk
*      endif
*   endfunc
*
*   DumpToExcel(,,,,"my_init", "my_increment", "my_difference")
function DumpToExcel
parameters m.dbName, m.R, m.mc, m.format, m.funcinit, m.funcincr, m.funcdiff, m.SkipTitles
	local m.DataFirstRow
	
	if type('m.dbName') == 'C'
		m.oldDbName = alias()
		sele (m.dbName)
	else
		if used() == .F.
			wait window "No database open, no database specified"
			return
		endif
	endif

	if type('m.Mc') == 'L'
		m.mc = StartExcelDDE()
	endif

	if type('m.R') != 'N'
		m.R = 1
	endif
	
	if type('m.format') != 'C'
		m.format = ""
	endif
	padr(m.format, fcount())
	
	if type('m.funcinit') != 'C'
		m.funcinit = ""
	else
		m.funcinit = m.funcinit + "()"
	endif

	if type('m.funcincr') != 'C'
		m.funcincr = ""
	else
		m.funcincr = m.funcincr + "()"
	endif

	if type('m.funcdiff') != 'C'
		m.funcdiff = ""
	else
		m.funcdiff = "m.R = " + m.funcdiff + "(m.mc, m.R)"
	endif

	* Draw titles
	dimension m.af[1]
	afields(m.af)
	if m.SkipTitles == .F. then
		for m.i = 1 to fcount()
			DDEPoke(m.mc, MakeRC(m.R, m.i), field(m.i))
		next
		ddeexecute(m.mc, '[SELECT("' + MakeRC(m.R, 1) + ':' + MakeRC(m.R, fcount()) + ;
			'")][ALIGNMENT(3)][FONT.PROPERTIES(,"Bold")][BORDER(,,,,6)]')
		m.R = m.R + 1
		go top
	endif
	
	* Dump the data
	&funcinit
	m.DataFirstRow = m.R
	do while ! eof()
		for m.i = 1 to fcount()
			m.d = af(m.i,1)
			m.d = &d
			if af(m.i, 2) == 'N' or af(m.i, 2) == 'Y' then
				m.d = str(m.d, af(m.i, 3) + 1 + af(m.i, 4), af(m.i, 4))
			endif
			if af(m.i, 2) == 'I' then
				m.d = str(m.d)
			endif
			if af(m.i, 2) == 'M' then
				m.d = substr(m.d, 1)
			endif
			if af(m.i, 2) == 'L' then
				if m.d == .T. then
					m.d = "1"
				else
					m.d = "0"
				endif
			endif
			if af(m.i, 2) == 'D' then
				m.d = dtoc(m.d)
			endif
			DDEPoke(mc, MakeRC(m.R, m.i), allt(m.d))
		next
		&funcincr
		skip
		m.R = m.R + 1
		&funcdiff
	enddo
	
	* Set formatting
	for m.i = 1 to fcount()
		m.c = substr(m.format, m.i, 1)
		m.cmd = ""

		if (m.c == "0")
			m.cmd = 'FORMAT.NUMBER("#,##0")'
		endif
		if (m.c == "1")
			m.cmd = 'FORMAT.NUMBER("#,##0.0")'
		endif
		if (m.c == "2")
			m.cmd = 'FORMAT.NUMBER("#,##0.00")'
		endif
		if (m.c == "3")
			m.cmd = 'FORMAT.NUMBER("#,##0.000")'
		endif
		if (m.c == "4")
			m.cmd = 'FORMAT.NUMBER("#,##0.0000")'
		endif
		if (m.c == "5")
			m.cmd = 'FORMAT.NUMBER("#,##0.00000")'
		endif
		if (m.c == "6")
			m.cmd = 'FORMAT.NUMBER("#,##0.000000")'
		endif
		if (m.c == "7")
			m.cmd = 'FORMAT.NUMBER("#,##0.0000000")'
		endif
		if (m.c == "8")
			m.cmd = 'FORMAT.NUMBER("#,##0.00000000")'
		endif
		if (m.c == "9")
			m.cmd = 'FORMAT.NUMBER("#,##0.000000000")'
		endif
		if (m.c == "$")
			m.cmd = 'FORMAT.NUMBER("$#,##0.00")'
		endif
		
		if ! empty(m.cmd)
			ddeexecute(m.mc, '[SELECT("' + MakeRC(m.DataFirstRow, m.i) + ':' + ;
				MakeRC(m.R, m.i) + '")][' + m.cmd + ']')
		endif
	next
	ddeexecute(mc, '[SELECT("R1C1")]')

	* Auto width
	ddeexecute(mc, '[SELECT("R1C1")]')
	ddeexecute(mc, '[COLUMN.WIDTH(,"R1C1:' + MakeRC(R, fcount()) + '",,3)]')
	
	if type('m.dbName') == 'C'
		sele (m.oldDbName)
	endif
	
	return m.mc
endfunc
